@@ -504,6 +504,17 @@ static unsigned exam_paranoia20()
 	return result;
 }
 
+static unsigned exam_mul_info()
+{
+	symbol x("x"), y("y");
+	ex e = x*y;
+	if (!e.info(info_flags::indefinite)) {
+		clog << "eek, product of two symbols is NOT indefinite\n";
+		return 1;
+	}
+	return 0;
+}
+
 static unsigned is_polynomial_false_positive()
 {
 	unsigned result = 0;
@@ -636,6 +647,43 @@ unsigned exam_paranoia25()
 	return 0;
 }
 
+// Bug in partial fraction expansion
+unsigned exam_paranoia26()
+{
+	symbol x("x");
+	ex ex1=pow(x,4)/(x-1)/4;
+	ex ex2=sqrfree_parfrac(ex1,x);
+	ex e = (ex1-ex2).normal();
+
+	if (! e.is_zero()) {
+		clog << "partial fraction expansion of " << ex1 << " produces error.\n";
+		return 1;
+	}
+	return 0;
+}
+
+// Bug in power expansion
+unsigned exam_paranoia27()
+{
+	unsigned result = 0;
+	symbol x("x"), y("y"), a("a");
+	possymbol s("s"), t("t");
+	exmap pwrs =
+	  { {pow((x+1)*(y-2)*(s-3)*(t+4), a), pow((x+1)*(y-2)*(s-3), a)*pow(t+4, a)},
+	    {pow(2*(x+1)*(y-2)*(s-3)*(t+4), a), pow(2,a)*pow((x+1)*(y-2)*(s-3), a)*pow(t+4, a)},
+	    {pow(-(x+1)*(y-2)*(s-3)*(t+4), a), pow(-(x+1)*(y-2)*(s-3), a)*pow(t+4, a)},
+	    {pow(-2*(x+1)*(y-2)*(s-3)*(t+4), a), pow(2,a)*pow(-(x+1)*(y-2)*(s-3), a)*pow(t+4, a)} };
+
+	for (auto e : pwrs) {
+		if (! (e.first.expand()).is_equal(e.second) ) {
+			clog << "power expansion of " << e.first << " produces error.\n";
+			++result;
+		}
+	}
+
+	return result;
+}
+
 unsigned exam_paranoia()
 {
 	unsigned result = 0;
@@ -662,12 +710,15 @@ unsigned exam_paranoia()
 	result += exam_paranoia18();  cout << '.' << flush;
 	result += exam_paranoia19();  cout << '.' << flush;
 	result += exam_paranoia20();  cout << '.' << flush;
+	result += exam_mul_info(); cout << '.' << flush;
 	result += is_polynomial_false_positive(); cout << '.' << flush;
 	result += exam_paranoia21();  cout << '.' << flush;
 	result += exam_paranoia22();  cout << '.' << flush;
 	result += exam_paranoia23();  cout << '.' << flush;
 	result += exam_paranoia24();  cout << '.' << flush;
 	result += exam_paranoia25();  cout << '.' << flush;
+	result += exam_paranoia26();  cout << '.' << flush;
+	result += exam_paranoia27();  cout << '.' << flush;
 	
 	return result;
 }
