@@ -3,7 +3,7 @@
  *  Interface to GiNaC's clifford algebra (Dirac gamma) objects. */
 
 /*
- *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2020 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -46,21 +46,21 @@ public:
 	clifford(const ex & b, const ex & mu,  const ex & metr, unsigned char rl = 0, int comm_sign = -1);
 
 	// internal constructors
-	clifford(unsigned char rl, const ex & metr, int comm_sign, const exvector & v, bool discardable = false);
-	clifford(unsigned char rl, const ex & metr, int comm_sign, std::auto_ptr<exvector> vp);
+	clifford(unsigned char rl, const ex & metr, int comm_sign, const exvector & v);
+	clifford(unsigned char rl, const ex & metr, int comm_sign, exvector && v);
 
 	// functions overriding virtual functions from base classes
 public:
-	unsigned precedence() const { return 65; }
-	void archive(archive_node& n) const;
-	void read_archive(const archive_node& n, lst& sym_lst);
+	unsigned precedence() const override { return 65; }
+	void archive(archive_node& n) const override;
+	void read_archive(const archive_node& n, lst& sym_lst) override;
 protected:
-	ex eval_ncmul(const exvector & v) const;
-	bool match_same_type(const basic & other) const;
-	ex thiscontainer(const exvector & v) const;
-	ex thiscontainer(std::auto_ptr<exvector> vp) const;
-	unsigned return_type() const { return return_types::noncommutative; }
-	return_type_t return_type_tinfo() const;
+	ex eval_ncmul(const exvector & v) const override;
+	bool match_same_type(const basic & other) const override;
+	ex thiscontainer(const exvector & v) const override;
+	ex thiscontainer(exvector && v) const override;
+	unsigned return_type() const override { return return_types::noncommutative; }
+	return_type_t return_type_tinfo() const override;
 	// non-virtual functions in this class
 public:
 	unsigned char get_representation_label() const { return representation_label; }
@@ -69,14 +69,15 @@ public:
 	bool same_metric(const ex & other) const;
 	int get_commutator_sign() const { return commutator_sign; } //**< See the member variable commutator_sign */
 
-	inline size_t nops() const {return inherited::nops() + 1; }
-	ex op(size_t i) const;
-	ex & let_op(size_t i);
-	ex subs(const exmap & m, unsigned options = 0) const;
+	inline size_t nops() const override {return inherited::nops() + 1; }
+	ex op(size_t i) const override;
+	ex & let_op(size_t i) override;
+	ex subs(const exmap & m, unsigned options = 0) const override;
 
 protected:
 	void do_print_dflt(const print_dflt & c, unsigned level) const;
 	void do_print_latex(const print_latex & c, unsigned level) const;
+	void do_print_tree(const print_tree & c, unsigned level) const;
 
 	// member variables
 protected:
@@ -106,13 +107,14 @@ class cliffordunit : public tensor
 
 	// functions overriding virtual functions from base classes
 public:
-	bool contract_with(exvector::iterator self, exvector::iterator other, exvector & v) const;
+	bool contract_with(exvector::iterator self, exvector::iterator other, exvector & v) const override;
 
 	// non-virtual functions in this class
 protected:
 	void do_print(const print_context & c, unsigned level) const;
 	void do_print_latex(const print_latex & c, unsigned level) const;
 };
+GINAC_DECLARE_UNARCHIVER(cliffordunit);
 
 
 /** This class represents the Dirac gamma Lorentz vector. */
@@ -122,7 +124,7 @@ class diracgamma : public cliffordunit
 
 	// functions overriding virtual functions from base classes
 public:
-	bool contract_with(exvector::iterator self, exvector::iterator other, exvector & v) const;
+	bool contract_with(exvector::iterator self, exvector::iterator other, exvector & v) const override;
 
 	// non-virtual functions in this class
 protected:
@@ -139,7 +141,7 @@ class diracgamma5 : public tensor
 	GINAC_DECLARE_REGISTERED_CLASS(diracgamma5, tensor)
 
 	// functions overriding virtual functions from base classes
-	ex conjugate() const;
+	ex conjugate() const override;
 
 	// non-virtual functions in this class
 protected:
@@ -156,7 +158,7 @@ class diracgammaL : public tensor
 	GINAC_DECLARE_REGISTERED_CLASS(diracgammaL, tensor)
 
 	// functions overriding virtual functions from base classes
-	ex conjugate() const;
+	ex conjugate() const override;
 
 	// non-virtual functions in this class
 protected:
@@ -173,7 +175,7 @@ class diracgammaR : public tensor
 	GINAC_DECLARE_REGISTERED_CLASS(diracgammaR, tensor)
 
 	// functions overriding virtual functions from base classes
-	ex conjugate() const;
+	ex conjugate() const override;
 
 	// non-virtual functions in this class
 protected:
@@ -280,12 +282,16 @@ ex canonicalize_clifford(const ex & e);
  *  clifford units. */
 ex clifford_prime(const ex & e);
 
+/** An auxillary function performing clifford_star() and clifford_bar().*/
+ex clifford_star_bar(const ex & e, bool do_bar, unsigned options);
+
 /** Main anti-automorphism of the Clifford algebra: makes reversion
  *  and changes signs of all clifford units. */
-inline ex clifford_bar(const ex & e) { return clifford_prime(e.conjugate()); }
+inline ex clifford_bar(const ex & e) { return clifford_star_bar(e, true, 0); }
 
-/** Reversion of the Clifford algebra, coincides with the conjugate(). */
-inline ex clifford_star(const ex & e) { return e.conjugate(); }
+/** Reversion of the Clifford algebra, reverse the order of all clifford units
+ *  in ncmul. */
+inline ex clifford_star(const ex & e) { return clifford_star_bar(e, false, 0); }
 
 /** Replaces dirac_ONE's (with a representation_label no less than rl) in e with 1.
  *  For the default value rl = 0 remove all of them. Aborts if e contains any 
@@ -301,7 +307,7 @@ ex remove_dirac_ONE(const ex & e, unsigned char rl = 0, unsigned options = 0);
  *
  *  @param e Expression to be processed
  *  @ignore_ONE defines if clifford_ONE should be ignored in the search*/
-char clifford_max_label(const ex & e, bool ignore_ONE = false);
+int clifford_max_label(const ex & e, bool ignore_ONE = false);
 
 /** Calculation of the norm in the Clifford algebra. */
 ex clifford_norm(const ex & e);
