@@ -3,7 +3,7 @@
  *  Wrapper template for making GiNaC classes out of STL containers. */
 
 /*
- *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2020 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include "print.h"
 #include "archive.h"
 #include "assertion.h"
+#include "compiler.h"
 
 #include <algorithm>
 #include <iterator>
@@ -38,13 +39,14 @@
 namespace GiNaC {
 
 /** Helper template for encapsulating the reserve() mechanics of STL containers. */
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 class container_storage {
 protected:
 	typedef C<ex> STLT;
 
 	container_storage() {}
 	container_storage(size_t n, const ex & e) : seq(n, e) {}
+	container_storage(std::initializer_list<ex> il) : seq(il) {}
 
 	template <class In>
 	container_storage(In b, In e) : seq(b, e) {}
@@ -124,7 +126,7 @@ private:
 };
 
 /** Wrapper template for making GiNaC classes out of STL containers. */
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 class container : public basic, public container_storage<C> {
 	GINAC_DECLARE_REGISTERED_CLASS(container, basic)
 protected:
@@ -142,20 +144,16 @@ protected:
 
 	// constructors
 public:
-	container(STLT const & s, bool discardable = false)
+	container(STLT const & s)
 	{
 		setflag(get_default_flags());
-
-		if (discardable)
-			this->seq.swap(const_cast<STLT &>(s));
-		else
-			this->seq = s;
+		this->seq = s;
 	}
 
-	explicit container(std::auto_ptr<STLT> vp)
+	explicit container(STLT && v)
 	{
 		setflag(get_default_flags());
-		this->seq.swap(*vp);
+		this->seq.swap(v);
 	}
 
 	container(exvector::const_iterator b, exvector::const_iterator e)
@@ -164,237 +162,78 @@ public:
 		setflag(get_default_flags());
 	}
 
-	explicit container(const ex & p1)
-	 : container_storage<C>(1, p1)
+	container(std::initializer_list<ex> il)
+	 : container_storage<C>(il)
 	{
 		setflag(get_default_flags());
 	}
 
-	container(const ex & p1, const ex & p2)
-	{
-		setflag(get_default_flags());
-		this->reserve(this->seq, 2);
-		this->seq.push_back(p1); this->seq.push_back(p2);
-	}
-
-	container(const ex & p1, const ex & p2, const ex & p3)
-	{
-		setflag(get_default_flags());
-		this->reserve(this->seq, 3);
-		this->seq.push_back(p1); this->seq.push_back(p2); this->seq.push_back(p3);
-	}
-
-	container(const ex & p1, const ex & p2, const ex & p3,
-	          const ex & p4)
-	{
-		setflag(get_default_flags());
-		this->reserve(this->seq, 4);
-		this->seq.push_back(p1); this->seq.push_back(p2); this->seq.push_back(p3);
-		this->seq.push_back(p4);
-	}
-
-	container(const ex & p1, const ex & p2, const ex & p3,
-	          const ex & p4, const ex & p5)
-	{
-		setflag(get_default_flags());
-		this->reserve(this->seq, 5);
-		this->seq.push_back(p1); this->seq.push_back(p2); this->seq.push_back(p3);
-		this->seq.push_back(p4); this->seq.push_back(p5);
-	}
-
-	container(const ex & p1, const ex & p2, const ex & p3,
-	          const ex & p4, const ex & p5, const ex & p6)
-	{
-		setflag(get_default_flags());
-		this->reserve(this->seq, 6);
-		this->seq.push_back(p1); this->seq.push_back(p2); this->seq.push_back(p3);
-		this->seq.push_back(p4); this->seq.push_back(p5); this->seq.push_back(p6);
-	}
-
-	container(const ex & p1, const ex & p2, const ex & p3,
-	          const ex & p4, const ex & p5, const ex & p6,
-	          const ex & p7)
-	{
-		setflag(get_default_flags());
-		this->reserve(this->seq, 7);
-		this->seq.push_back(p1); this->seq.push_back(p2); this->seq.push_back(p3);
-		this->seq.push_back(p4); this->seq.push_back(p5); this->seq.push_back(p6);
-		this->seq.push_back(p7);
-	}
-
-	container(const ex & p1, const ex & p2, const ex & p3,
-	          const ex & p4, const ex & p5, const ex & p6,
-	          const ex & p7, const ex & p8)
-	{
-		setflag(get_default_flags());
-		this->reserve(this->seq, 8);
-		this->seq.push_back(p1); this->seq.push_back(p2); this->seq.push_back(p3);
-		this->seq.push_back(p4); this->seq.push_back(p5); this->seq.push_back(p6);
-		this->seq.push_back(p7); this->seq.push_back(p8);
-	}
-
-	container(const ex & p1, const ex & p2, const ex & p3,
-	          const ex & p4, const ex & p5, const ex & p6,
-	          const ex & p7, const ex & p8, const ex & p9)
-	{
-		setflag(get_default_flags());
-		this->reserve(this->seq, 9);
-		this->seq.push_back(p1); this->seq.push_back(p2); this->seq.push_back(p3);
-		this->seq.push_back(p4); this->seq.push_back(p5); this->seq.push_back(p6);
-		this->seq.push_back(p7); this->seq.push_back(p8); this->seq.push_back(p9);
-	}
-
-	container(const ex & p1, const ex & p2, const ex & p3,
-	          const ex & p4, const ex & p5, const ex & p6,
-	          const ex & p7, const ex & p8, const ex & p9,
-	          const ex & p10)
-	{
-		setflag(get_default_flags());
-		this->reserve(this->seq, 10);
-		this->seq.push_back(p1); this->seq.push_back(p2); this->seq.push_back(p3);
-		this->seq.push_back(p4); this->seq.push_back(p5); this->seq.push_back(p6);
-		this->seq.push_back(p7); this->seq.push_back(p8); this->seq.push_back(p9);
-		this->seq.push_back(p10);
-	}
-
-	container(const ex & p1, const ex & p2, const ex & p3,
-	          const ex & p4, const ex & p5, const ex & p6,
-	          const ex & p7, const ex & p8, const ex & p9,
-	          const ex & p10, const ex & p11)
-	{
-		setflag(get_default_flags());
-		this->reserve(this->seq, 11);
-		this->seq.push_back(p1); this->seq.push_back(p2); this->seq.push_back(p3);
-		this->seq.push_back(p4); this->seq.push_back(p5); this->seq.push_back(p6);
-		this->seq.push_back(p7); this->seq.push_back(p8); this->seq.push_back(p9);
-		this->seq.push_back(p10); this->seq.push_back(p11);
-	}
-
-	container(const ex & p1, const ex & p2, const ex & p3,
-	          const ex & p4, const ex & p5, const ex & p6,
-	          const ex & p7, const ex & p8, const ex & p9,
-	          const ex & p10, const ex & p11, const ex & p12)
-	{
-		setflag(get_default_flags());
-		this->reserve(this->seq, 12);
-		this->seq.push_back(p1); this->seq.push_back(p2); this->seq.push_back(p3);
-		this->seq.push_back(p4); this->seq.push_back(p5); this->seq.push_back(p6);
-		this->seq.push_back(p7); this->seq.push_back(p8); this->seq.push_back(p9);
-		this->seq.push_back(p10); this->seq.push_back(p11); this->seq.push_back(p12);
-	}
-
-	container(const ex & p1, const ex & p2, const ex & p3,
-	          const ex & p4, const ex & p5, const ex & p6,
-	          const ex & p7, const ex & p8, const ex & p9,
-	          const ex & p10, const ex & p11, const ex & p12,
-	          const ex & p13)
-	{
-		setflag(get_default_flags());
-		this->reserve(this->seq, 13);
-		this->seq.push_back(p1); this->seq.push_back(p2); this->seq.push_back(p3);
-		this->seq.push_back(p4); this->seq.push_back(p5); this->seq.push_back(p6);
-		this->seq.push_back(p7); this->seq.push_back(p8); this->seq.push_back(p9);
-		this->seq.push_back(p10); this->seq.push_back(p11); this->seq.push_back(p12);
-		this->seq.push_back(p13);
-	}
-
-	container(const ex & p1, const ex & p2, const ex & p3,
-	          const ex & p4, const ex & p5, const ex & p6,
-	          const ex & p7, const ex & p8, const ex & p9,
-	          const ex & p10, const ex & p11, const ex & p12,
-	          const ex & p13, const ex & p14)
-	{
-		setflag(get_default_flags());
-		this->reserve(this->seq, 14);
-		this->seq.push_back(p1); this->seq.push_back(p2); this->seq.push_back(p3);
-		this->seq.push_back(p4); this->seq.push_back(p5); this->seq.push_back(p6);
-		this->seq.push_back(p7); this->seq.push_back(p8); this->seq.push_back(p9);
-		this->seq.push_back(p10); this->seq.push_back(p11); this->seq.push_back(p12);
-		this->seq.push_back(p13); this->seq.push_back(p14);
-	}
-
-	container(const ex & p1, const ex & p2, const ex & p3,
-	          const ex & p4, const ex & p5, const ex & p6,
-	          const ex & p7, const ex & p8, const ex & p9,
-	          const ex & p10, const ex & p11, const ex & p12,
-	          const ex & p13, const ex & p14, const ex & p15)
-	{
-		setflag(get_default_flags());
-		this->reserve(this->seq, 15);
-		this->seq.push_back(p1); this->seq.push_back(p2); this->seq.push_back(p3);
-		this->seq.push_back(p4); this->seq.push_back(p5); this->seq.push_back(p6);
-		this->seq.push_back(p7); this->seq.push_back(p8); this->seq.push_back(p9);
-		this->seq.push_back(p10); this->seq.push_back(p11); this->seq.push_back(p12);
-		this->seq.push_back(p13); this->seq.push_back(p14); this->seq.push_back(p15);
-	}
-
-	container(const ex & p1, const ex & p2, const ex & p3,
-	          const ex & p4, const ex & p5, const ex & p6,
-	          const ex & p7, const ex & p8, const ex & p9,
-	          const ex & p10, const ex & p11, const ex & p12,
-	          const ex & p13, const ex & p14, const ex & p15,
-	          const ex & p16)
-	{
-		setflag(get_default_flags());
-		this->reserve(this->seq, 16);
-		this->seq.push_back(p1); this->seq.push_back(p2); this->seq.push_back(p3);
-		this->seq.push_back(p4); this->seq.push_back(p5); this->seq.push_back(p6);
-		this->seq.push_back(p7); this->seq.push_back(p8); this->seq.push_back(p9);
-		this->seq.push_back(p10); this->seq.push_back(p11); this->seq.push_back(p12);
-		this->seq.push_back(p13); this->seq.push_back(p14); this->seq.push_back(p15);
-		this->seq.push_back(p16);
-	}
+	explicit container(const ex & p1) attribute_deprecated;
+	container(const ex & p1, const ex & p2) attribute_deprecated;
+	container(const ex & p1, const ex & p2, const ex & p3) attribute_deprecated;
+	container(const ex & p1, const ex & p2, const ex & p3, const ex & p4) attribute_deprecated;
+	container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5) attribute_deprecated;
+	container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6) attribute_deprecated;
+	container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7) attribute_deprecated;
+	container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8) attribute_deprecated;
+	container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8,
+	          const ex & p9) attribute_deprecated;
+	container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8,
+	          const ex & p9, const ex & p10) attribute_deprecated;
+	container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8,
+	          const ex & p9, const ex & p10, const ex & p11) attribute_deprecated;
+	container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8,
+	          const ex & p9, const ex & p10, const ex & p11, const ex & p12) attribute_deprecated;
+	container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8,
+	          const ex & p9, const ex & p10, const ex & p11, const ex & p12, const ex & p13) attribute_deprecated;
+	container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8,
+	          const ex & p9, const ex & p10, const ex & p11, const ex & p12, const ex & p13, const ex & p14) attribute_deprecated;
+	container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8,
+	          const ex & p9, const ex & p10, const ex & p11, const ex & p12, const ex & p13, const ex & p14, const ex & p15) attribute_deprecated;
+	container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8,
+	          const ex & p9, const ex & p10, const ex & p11, const ex & p12, const ex & p13, const ex & p14, const ex & p15, const ex & p16) attribute_deprecated;
 
 	// First step of initialization of container with a comma-separated
 	// sequence of expressions. Subsequent steps are handled by
 	// container_init<>::operator,().
-	container_init<ex, STLT> operator=(const ex & x)
-	{
-		this->seq.push_back(x);
-		return container_init<ex, STLT>(this->seq);
-	}
+	container_init<ex, STLT> operator=(const ex & x) attribute_deprecated;
 
 	// functions overriding virtual functions from base classes
 public:
-	bool info(unsigned inf) const { return inherited::info(inf); }
-	unsigned precedence() const { return 10; }
-	size_t nops() const { return this->seq.size(); }
-	ex op(size_t i) const;
-	ex & let_op(size_t i);
-	ex eval(int level = 0) const;
-	ex subs(const exmap & m, unsigned options = 0) const;
+	bool info(unsigned inf) const override { return inherited::info(inf); }
+	unsigned precedence() const override { return 10; }
+	size_t nops() const override { return this->seq.size(); }
+	ex op(size_t i) const override;
+	ex & let_op(size_t i) override;
+	ex subs(const exmap & m, unsigned options = 0) const override;
 
-	void read_archive(const archive_node &n, lst &sym_lst) 
+	void read_archive(const archive_node &n, lst &sym_lst) override
 	{
 		inherited::read_archive(n, sym_lst);
 		setflag(get_default_flags());
 
-		archive_node::archive_node_cit first = n.find_first("seq");
-		archive_node::archive_node_cit last = n.find_last("seq");
-		++last;
-		this->reserve(this->seq, last - first);
-		for (archive_node::archive_node_cit i=first; i<last; ++i) {
+		auto range =  n.find_property_range("seq", "seq");
+		this->reserve(this->seq, range.end - range.begin);
+		for (archive_node::archive_node_cit i=range.begin; i<range.end; ++i) {
 			ex e;
 			n.find_ex_by_loc(i, e, sym_lst);
-			this->seq.push_back(e);
+			this->seq.emplace_back(e);
 		}
 	}
 
 	/** Archive the object. */
-	void archive(archive_node &n) const
+	void archive(archive_node &n) const override
 	{
 		inherited::archive(n);
-		const_iterator i = this->seq.begin(), end = this->seq.end();
-		while (i != end) {
-			n.add_ex("seq", *i);
-			++i;
+		for (auto & i : this->seq) {
+			n.add_ex("seq", i);
 		}
 	}
 
 protected:
-	ex conjugate() const
+	ex conjugate() const override
 	{
-		STLT *newcont = NULL;
+		STLT *newcont = nullptr;
 		for (const_iterator i=this->seq.begin(); i!=this->seq.end(); ++i) {
 			if (newcont) {
 				newcont->push_back(i->conjugate());
@@ -419,7 +258,7 @@ protected:
 		return *this;
 	}
 
-	ex real_part() const
+	ex real_part() const override
 	{
 		STLT cont;
 		this->reserve(cont, nops());
@@ -430,7 +269,7 @@ protected:
 		return thiscontainer(cont);
 	}
 
-	ex imag_part() const
+	ex imag_part() const override
 	{
 		STLT cont;
 		this->reserve(cont, nops());
@@ -441,7 +280,7 @@ protected:
 		return thiscontainer(cont);
 	}
 
-	bool is_equal_same_type(const basic & other) const;
+	bool is_equal_same_type(const basic & other) const override;
 
 	// new virtual functions which can be overridden by derived classes
 protected:
@@ -450,8 +289,8 @@ protected:
 	virtual ex thiscontainer(const STLT & v) const { return container(v); }
 
 	/** Similar to duplicate(), but with a preset sequence (which gets
-	 *  deleted). Must be overridden by derived classes. */
-	virtual ex thiscontainer(std::auto_ptr<STLT> vp) const { return container(vp); }
+	 *  pilfered). Must be overridden by derived classes. */
+	virtual ex thiscontainer(STLT && v) const { return container(std::move(v)); }
 
 	virtual void printseq(const print_context & c, char openbracket, char delim,
 	                      char closebracket, unsigned this_precedence,
@@ -494,26 +333,103 @@ protected:
 	void do_print_tree(const print_tree & c, unsigned level) const;
 	void do_print_python(const print_python & c, unsigned level) const;
 	void do_print_python_repr(const print_python_repr & c, unsigned level) const;
-	STLT evalchildren(int level) const;
-	std::auto_ptr<STLT> subschildren(const exmap & m, unsigned options = 0) const;
+	STLT subschildren(const exmap & m, unsigned options = 0) const;
 };
 
 /** Default constructor */
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 container<C>::container()
 {
 	setflag(get_default_flags());
 }
 
+/** Deprecatd constructors (prefer initializer list) */
+template <template <class T, class = std::allocator<T>> class C>
+container<C>::container(const ex & p1)
+  : container_storage<C>(1, p1) { setflag(get_default_flags()); }
 
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
+container<C>::container(const ex & p1, const ex & p2)
+  : container_storage<C>{p1, p2} { setflag(get_default_flags()); }
+
+template <template <class T, class = std::allocator<T>> class C>
+container<C>::container(const ex & p1, const ex & p2, const ex & p3)
+  : container_storage<C>{p1, p2, p3} { setflag(get_default_flags()); }
+
+template <template <class T, class = std::allocator<T>> class C>
+container<C>::container(const ex & p1, const ex & p2, const ex & p3, const ex & p4)
+  : container_storage<C>{p1, p2, p3, p4} { setflag(get_default_flags()); }
+
+template <template <class T, class = std::allocator<T>> class C>
+container<C>::container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5)
+  : container_storage<C>{p1, p2, p3, p4, p5} { setflag(get_default_flags()); }
+
+template <template <class T, class = std::allocator<T>> class C>
+container<C>::container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6)
+  : container_storage<C>{p1, p2, p3, p4, p5, p6} { setflag(get_default_flags()); }
+
+template <template <class T, class = std::allocator<T>> class C>
+container<C>::container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7)
+  : container_storage<C>{p1, p2, p3, p4, p5, p6, p7} { setflag(get_default_flags()); }
+
+template <template <class T, class = std::allocator<T>> class C>
+container<C>::container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8)
+  : container_storage<C>{p1, p2, p3, p4, p5, p6, p7, p8} { setflag(get_default_flags()); }
+
+template <template <class T, class = std::allocator<T>> class C>
+container<C>::container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8,
+                        const ex & p9)
+  : container_storage<C>{p1, p2, p3, p4, p5, p6, p7, p8, p9} { setflag(get_default_flags()); }
+
+template <template <class T, class = std::allocator<T>> class C>
+container<C>::container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8,
+                        const ex & p9, const ex & p10)
+  : container_storage<C>{p1, p2, p3, p4, p5, p6, p7, p8, p9, p10} { setflag(get_default_flags()); }
+
+template <template <class T, class = std::allocator<T>> class C>
+container<C>::container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8,
+                        const ex & p9, const ex & p10, const ex & p11)
+  : container_storage<C>{p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11} { setflag(get_default_flags()); }
+
+template <template <class T, class = std::allocator<T>> class C>
+container<C>::container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8,
+                        const ex & p9, const ex & p10, const ex & p11, const ex & p12)
+  : container_storage<C>{p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12} { setflag(get_default_flags()); }
+
+template <template <class T, class = std::allocator<T>> class C>
+container<C>::container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8,
+                        const ex & p9, const ex & p10, const ex & p11, const ex & p12, const ex & p13)
+  : container_storage<C>{p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13} { setflag(get_default_flags()); }
+
+template <template <class T, class = std::allocator<T>> class C>
+container<C>::container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8,
+                        const ex & p9, const ex & p10, const ex & p11, const ex & p12, const ex & p13, const ex & p14)
+  : container_storage<C>{p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14} { setflag(get_default_flags()); }
+
+template <template <class T, class = std::allocator<T>> class C>
+container<C>::container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8,
+                        const ex & p9, const ex & p10, const ex & p11, const ex & p12, const ex & p13, const ex & p14, const ex & p15)
+  : container_storage<C>{p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15} { setflag(get_default_flags()); }
+template <template <class T, class = std::allocator<T>> class C>
+container<C>::container(const ex & p1, const ex & p2, const ex & p3, const ex & p4, const ex & p5, const ex & p6, const ex & p7, const ex & p8,
+                        const ex & p9, const ex & p10, const ex & p11, const ex & p12, const ex & p13, const ex & p14, const ex & p15, const ex & p16)
+  : container_storage<C>{p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15, p16} { setflag(get_default_flags()); }
+
+template <template <class T, class = std::allocator<T>> class C>
+container_init<ex, typename container_storage<C>::STLT> container<C>::operator=(const ex & x)
+{
+	this->seq.push_back(x);
+	return container_init<ex, typename container_storage<C>::STLT>(this->seq);
+}
+
+template <template <class T, class = std::allocator<T>> class C>
 void container<C>::do_print(const print_context & c, unsigned level) const
 {
 	// always print brackets around seq, ignore upper_precedence
 	printseq(c, get_open_delim(), ',', get_close_delim(), precedence(), precedence()+1);
 }
 
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 void container<C>::do_print_tree(const print_tree & c, unsigned level) const
 {
 	c.s << std::string(level, ' ') << class_name() << " @" << this
@@ -528,20 +444,20 @@ void container<C>::do_print_tree(const print_tree & c, unsigned level) const
 	c.s << std::string(level + c.delta_indent,' ') << "=====" << std::endl;
 }
 
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 void container<C>::do_print_python(const print_python & c, unsigned level) const
 {
 	printseq(c, '[', ',', ']', precedence(), precedence()+1);
 }
 
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 void container<C>::do_print_python_repr(const print_python_repr & c, unsigned level) const
 {
 	c.s << class_name();
 	printseq(c, '(', ',', ')', precedence(), precedence()+1);
 }
 
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 ex container<C>::op(size_t i) const
 {
 	GINAC_ASSERT(i < nops());
@@ -551,7 +467,7 @@ ex container<C>::op(size_t i) const
 	return *it;
 }
 
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 ex & container<C>::let_op(size_t i)
 {
 	GINAC_ASSERT(i < nops());
@@ -562,16 +478,7 @@ ex & container<C>::let_op(size_t i)
 	return *it;
 }
 
-template <template <class T, class = std::allocator<T> > class C>
-ex container<C>::eval(int level) const
-{
-	if (level == 1)
-		return hold();
-	else
-		return thiscontainer(evalchildren(level));
-}
-
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 ex container<C>::subs(const exmap & m, unsigned options) const
 {
 	// After having subs'ed all children, this method subs'es one final
@@ -582,15 +489,15 @@ ex container<C>::subs(const exmap & m, unsigned options) const
 	// f(x).subs(x==f^-1(x))
 	//   -> f(f^-1(x))  [subschildren]
 	//   -> x           [eval]   /* must not subs(x==f^-1(x))! */
-	std::auto_ptr<STLT> vp = subschildren(m, options);
-	if (vp.get()) {
-		ex result(thiscontainer(vp));
-		if (is_a<container<C> >(result))
+	STLT subsed = subschildren(m, options);
+	if (!subsed.empty()) {
+		ex result(thiscontainer(subsed));
+		if (is_a<container<C>>(result))
 			return ex_to<basic>(result).subs_one_level(m, options);
 		else
 			return result;
 	} else {
-		if (is_a<container<C> >(*this))
+		if (is_a<container<C>>(*this))
 			return subs_one_level(m, options);
 		else
 			return *this;
@@ -598,7 +505,7 @@ ex container<C>::subs(const exmap & m, unsigned options) const
 }
 
 /** Compare two containers of the same type. */
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 int container<C>::compare_same_type(const basic & other) const
 {
 	GINAC_ASSERT(is_a<container>(other));
@@ -617,7 +524,7 @@ int container<C>::compare_same_type(const basic & other) const
 	return (it1 == it1end) ? (it2 == it2end ? 0 : -1) : 1;
 }
 
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 bool container<C>::is_equal_same_type(const basic & other) const
 {
 	GINAC_ASSERT(is_a<container>(other));
@@ -637,7 +544,7 @@ bool container<C>::is_equal_same_type(const basic & other) const
 }
 
 /** Add element at front. */
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 container<C> & container<C>::prepend(const ex & b)
 {
 	ensure_if_modifiable();
@@ -646,7 +553,7 @@ container<C> & container<C>::prepend(const ex & b)
 }
 
 /** Add element at back. */
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 container<C> & container<C>::append(const ex & b)
 {
 	ensure_if_modifiable();
@@ -655,7 +562,7 @@ container<C> & container<C>::append(const ex & b)
 }
 
 /** Remove first element. */
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 container<C> & container<C>::remove_first()
 {
 	ensure_if_modifiable();
@@ -664,7 +571,7 @@ container<C> & container<C>::remove_first()
 }
 
 /** Remove last element. */
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 container<C> & container<C>::remove_last()
 {
 	ensure_if_modifiable();
@@ -673,7 +580,7 @@ container<C> & container<C>::remove_last()
 }
 
 /** Remove all elements. */
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 container<C> & container<C>::remove_all()
 {
 	ensure_if_modifiable();
@@ -682,7 +589,7 @@ container<C> & container<C>::remove_all()
 }
 
 /** Sort elements. */
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 container<C> & container<C>::sort()
 {
 	ensure_if_modifiable();
@@ -697,7 +604,7 @@ template<> inline void container<std::list>::unique_()
 }
 
 /** Remove adjacent duplicate elements. */
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 container<C> & container<C>::unique()
 {
 	ensure_if_modifiable();
@@ -706,7 +613,7 @@ container<C> & container<C>::unique()
 }
 
 /** Print sequence of contained elements. */
-template <template <class T, class = std::allocator<T> > class C>
+template <template <class T, class = std::allocator<T>> class C>
 void container<C>::printseq(const print_context & c, char openbracket, char delim,
                             char closebracket, unsigned this_precedence,
                             unsigned upper_precedence) const
@@ -729,33 +636,11 @@ void container<C>::printseq(const print_context & c, char openbracket, char deli
 		c.s << closebracket;
 }
 
-template <template <class T, class = std::allocator<T> > class C>
-typename container<C>::STLT container<C>::evalchildren(int level) const
+template <template <class T, class = std::allocator<T>> class C>
+typename container<C>::STLT container<C>::subschildren(const exmap & m, unsigned options) const
 {
-	if (level == 1)
-		return this->seq;
-	else if (level == -max_recursion_level)
-		throw std::runtime_error("max recursion level reached");
-
-	STLT s;
-	this->reserve(s, this->seq.size());
-
-	--level;
-	const_iterator it = this->seq.begin(), itend = this->seq.end();
-	while (it != itend) {
-		s.push_back(it->eval(level));
-		++it;
-	}
-
-	return s;
-}
-
-template <template <class T, class = std::allocator<T> > class C>
-std::auto_ptr<typename container<C>::STLT> container<C>::subschildren(const exmap & m, unsigned options) const
-{
-	// returns a NULL pointer if nothing had to be substituted
-	// returns a pointer to a newly created STLT otherwise
-	// (and relinquishes responsibility for the STLT)
+	// returns an empty container if nothing had to be substituted
+	// returns a STLT with substituted elements otherwise
 
 	const_iterator cit = this->seq.begin(), end = this->seq.end();
 	while (cit != end) {
@@ -763,16 +648,16 @@ std::auto_ptr<typename container<C>::STLT> container<C>::subschildren(const exma
 		if (!are_ex_trivially_equal(*cit, subsed_ex)) {
 
 			// copy first part of seq which hasn't changed
-			std::auto_ptr<STLT> s(new STLT(this->seq.begin(), cit));
-			this->reserve(*s, this->seq.size());
+			STLT s(this->seq.begin(), cit);
+			this->reserve(s, this->seq.size());
 
 			// insert changed element
-			s->push_back(subsed_ex);
+			s.push_back(subsed_ex);
 			++cit;
 
 			// copy rest
 			while (cit != end) {
-				s->push_back(cit->subs(m, options));
+				s.push_back(cit->subs(m, options));
 				++cit;
 			}
 
@@ -782,7 +667,7 @@ std::auto_ptr<typename container<C>::STLT> container<C>::subschildren(const exma
 		++cit;
 	}
 	
-	return std::auto_ptr<STLT>(0); // nothing has changed
+	return STLT(); // nothing has changed
 }
 
 } // namespace GiNaC
