@@ -4,7 +4,7 @@
  *  methods for series expansion. */
 
 /*
- *  GiNaC Copyright (C) 1999-2019 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2020 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -394,17 +394,7 @@ ex pseries::collect(const ex &s, bool distributed) const
 /** Perform coefficient-wise automatic term rewriting rules in this class. */
 ex pseries::eval() const
 {
-	if (flags & status_flags::evaluated) {
-		return *this;
-	}
-
-	// Construct a new series with evaluated coefficients
-	epvector new_seq;
-	new_seq.reserve(seq.size());
-	for (auto & it : seq)
-		new_seq.emplace_back(expair(it.rest, it.coeff));
-
-	return dynallocate<pseries>(relational(var,point), std::move(new_seq)).setflag(status_flags::evaluated);
+	return hold();
 }
 
 /** Evaluate coefficients numerically. */
@@ -414,7 +404,7 @@ ex pseries::evalf() const
 	epvector new_seq;
 	new_seq.reserve(seq.size());
 	for (auto & it : seq)
-		new_seq.emplace_back(expair(it.rest, it.coeff));
+		new_seq.emplace_back(expair(it.rest.evalf(), it.coeff));
 
 	return dynallocate<pseries>(relational(var,point), std::move(new_seq)).setflag(status_flags::evaluated);
 }
