@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 3.6.3.  */
+/* A Bison parser, made by GNU Bison 3.7.  */
 
 /* Bison implementation for Yacc-like parsers in C
 
@@ -49,7 +49,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "3.6.3"
+#define YYBISON_VERSION "3.7"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -181,98 +181,7 @@ static void print_help_topics(void);
 #  endif
 # endif
 
-/* Use api.header.include to #include this header
-   instead of duplicating it here.  */
-#ifndef YY_YY_GINSH_PARSER_HPP_INCLUDED
-# define YY_YY_GINSH_PARSER_HPP_INCLUDED
-/* Debug traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 0
-#endif
-#if YYDEBUG
-extern int yydebug;
-#endif
-
-/* Token kinds.  */
-#ifndef YYTOKENTYPE
-# define YYTOKENTYPE
-  enum yytokentype
-  {
-    YYEMPTY = -2,
-    YYEOF = 0,                     /* "end of file"  */
-    YYerror = 256,                 /* error  */
-    YYUNDEF = 257,                 /* "invalid token"  */
-    T_NUMBER = 258,                /* T_NUMBER  */
-    T_SYMBOL = 259,                /* T_SYMBOL  */
-    T_LITERAL = 260,               /* T_LITERAL  */
-    T_DIGITS = 261,                /* T_DIGITS  */
-    T_QUOTE = 262,                 /* T_QUOTE  */
-    T_QUOTE2 = 263,                /* T_QUOTE2  */
-    T_QUOTE3 = 264,                /* T_QUOTE3  */
-    T_EQUAL = 265,                 /* T_EQUAL  */
-    T_NOTEQ = 266,                 /* T_NOTEQ  */
-    T_LESSEQ = 267,                /* T_LESSEQ  */
-    T_GREATEREQ = 268,             /* T_GREATEREQ  */
-    T_QUIT = 269,                  /* T_QUIT  */
-    T_WARRANTY = 270,              /* T_WARRANTY  */
-    T_PRINT = 271,                 /* T_PRINT  */
-    T_IPRINT = 272,                /* T_IPRINT  */
-    T_PRINTLATEX = 273,            /* T_PRINTLATEX  */
-    T_PRINTCSRC = 274,             /* T_PRINTCSRC  */
-    T_TIME = 275,                  /* T_TIME  */
-    T_XYZZY = 276,                 /* T_XYZZY  */
-    T_INVENTORY = 277,             /* T_INVENTORY  */
-    T_LOOK = 278,                  /* T_LOOK  */
-    T_SCORE = 279,                 /* T_SCORE  */
-    T_COMPLEX_SYMBOLS = 280,       /* T_COMPLEX_SYMBOLS  */
-    T_REAL_SYMBOLS = 281,          /* T_REAL_SYMBOLS  */
-    NEG = 282                      /* NEG  */
-  };
-  typedef enum yytokentype yytoken_kind_t;
-#endif
-/* Token kinds.  */
-#define YYEOF 0
-#define YYerror 256
-#define YYUNDEF 257
-#define T_NUMBER 258
-#define T_SYMBOL 259
-#define T_LITERAL 260
-#define T_DIGITS 261
-#define T_QUOTE 262
-#define T_QUOTE2 263
-#define T_QUOTE3 264
-#define T_EQUAL 265
-#define T_NOTEQ 266
-#define T_LESSEQ 267
-#define T_GREATEREQ 268
-#define T_QUIT 269
-#define T_WARRANTY 270
-#define T_PRINT 271
-#define T_IPRINT 272
-#define T_PRINTLATEX 273
-#define T_PRINTCSRC 274
-#define T_TIME 275
-#define T_XYZZY 276
-#define T_INVENTORY 277
-#define T_LOOK 278
-#define T_SCORE 279
-#define T_COMPLEX_SYMBOLS 280
-#define T_REAL_SYMBOLS 281
-#define NEG 282
-
-/* Value type.  */
-#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef int YYSTYPE;
-# define YYSTYPE_IS_TRIVIAL 1
-# define YYSTYPE_IS_DECLARED 1
-#endif
-
-
-extern YYSTYPE yylval;
-
-int yyparse (void);
-
-#endif /* !YY_YY_GINSH_PARSER_HPP_INCLUDED  */
+#include "ginsh_parser.hpp"
 /* Symbol kind.  */
 enum yysymbol_kind_t
 {
@@ -655,6 +564,7 @@ union yyalloc
 /* YYNSTATES -- Number of states.  */
 #define YYNSTATES  122
 
+/* YYMAXUTOK -- Last valid token kind.  */
 #define YYMAXUTOK   282
 
 
@@ -1143,7 +1053,7 @@ yydestruct (const char *yymsg,
 }
 
 
-/* The lookahead symbol.  */
+/* Lookahead token kind.  */
 int yychar;
 
 /* The semantic value of the lookahead symbol.  */
@@ -1161,34 +1071,30 @@ int yynerrs;
 int
 yyparse (void)
 {
-    yy_state_fast_t yystate;
+    yy_state_fast_t yystate = 0;
     /* Number of tokens to shift before error messages enabled.  */
-    int yyerrstatus;
+    int yyerrstatus = 0;
 
-    /* The stacks and their tools:
-       'yyss': related to states.
-       'yyvs': related to semantic values.
-
-       Refer to the stacks through separate pointers, to allow yyoverflow
+    /* Refer to the stacks through separate pointers, to allow yyoverflow
        to reallocate them elsewhere.  */
 
     /* Their size.  */
-    YYPTRDIFF_T yystacksize;
+    YYPTRDIFF_T yystacksize = YYINITDEPTH;
 
-    /* The state stack.  */
+    /* The state stack: array, bottom, top.  */
     yy_state_t yyssa[YYINITDEPTH];
-    yy_state_t *yyss;
-    yy_state_t *yyssp;
+    yy_state_t *yyss = yyssa;
+    yy_state_t *yyssp = yyss;
 
-    /* The semantic value stack.  */
+    /* The semantic value stack: array, bottom, top.  */
     YYSTYPE yyvsa[YYINITDEPTH];
-    YYSTYPE *yyvs;
-    YYSTYPE *yyvsp;
+    YYSTYPE *yyvs = yyvsa;
+    YYSTYPE *yyvsp = yyvs;
 
   int yyn;
   /* The return value of yyparse.  */
   int yyresult;
-  /* Lookahead token as an internal (translated) token number.  */
+  /* Lookahead symbol kind.  */
   yysymbol_kind_t yytoken = YYSYMBOL_YYEMPTY;
   /* The variables used to return semantic value and location from the
      action routines.  */
@@ -1202,15 +1108,6 @@ yyparse (void)
      Keep to zero when no symbol should be popped.  */
   int yylen = 0;
 
-  yynerrs = 0;
-  yystate = 0;
-  yyerrstatus = 0;
-
-  yystacksize = YYINITDEPTH;
-  yyssp = yyss = yyssa;
-  yyvsp = yyvs = yyvsa;
-
-
   YYDPRINTF ((stderr, "Starting parse\n"));
 
   yychar = YYEMPTY; /* Cause a token to be read.  */
@@ -1411,7 +1308,7 @@ yyreduce:
   YY_REDUCE_PRINT (yyn);
   switch (yyn)
     {
-  case 5:
+  case 5: /* line: exp ';'  */
 #line 151 "ginsh_parser.ypp"
                   {
 		try {
@@ -1422,10 +1319,10 @@ yyreduce:
 			YYERROR;
 		}
 	}
-#line 1426 "ginsh_parser.cpp"
+#line 1323 "ginsh_parser.cpp"
     break;
 
-  case 6:
+  case 6: /* line: exp ':'  */
 #line 160 "ginsh_parser.ypp"
                   {
 		try {
@@ -1435,10 +1332,10 @@ yyreduce:
 			YYERROR;
 		}
 	}
-#line 1439 "ginsh_parser.cpp"
+#line 1336 "ginsh_parser.cpp"
     break;
 
-  case 7:
+  case 7: /* line: T_PRINT '(' exp ')' ';'  */
 #line 168 "ginsh_parser.ypp"
                                   {
 		try {
@@ -1448,10 +1345,10 @@ yyreduce:
 			YYERROR;
 		}
 	}
-#line 1452 "ginsh_parser.cpp"
+#line 1349 "ginsh_parser.cpp"
     break;
 
-  case 8:
+  case 8: /* line: T_IPRINT '(' exp ')' ';'  */
 #line 176 "ginsh_parser.ypp"
                                    {
 		try {
@@ -1467,10 +1364,10 @@ yyreduce:
 			YYERROR;
 		}
 	}
-#line 1471 "ginsh_parser.cpp"
+#line 1368 "ginsh_parser.cpp"
     break;
 
-  case 9:
+  case 9: /* line: T_PRINTLATEX '(' exp ')' ';'  */
 #line 190 "ginsh_parser.ypp"
                                        {
 		try {
@@ -1480,10 +1377,10 @@ yyreduce:
 			YYERROR;
 		}
 	}
-#line 1484 "ginsh_parser.cpp"
+#line 1381 "ginsh_parser.cpp"
     break;
 
-  case 10:
+  case 10: /* line: T_PRINTCSRC '(' exp ')' ';'  */
 #line 198 "ginsh_parser.ypp"
                                       {
 		try {
@@ -1493,58 +1390,58 @@ yyreduce:
 			YYERROR;
 		}
 	}
-#line 1497 "ginsh_parser.cpp"
+#line 1394 "ginsh_parser.cpp"
     break;
 
-  case 11:
+  case 11: /* line: '?' T_SYMBOL  */
 #line 206 "ginsh_parser.ypp"
                                 {print_help(ex_to<symbol>(yyvsp[0]).get_name());}
-#line 1503 "ginsh_parser.cpp"
+#line 1400 "ginsh_parser.cpp"
     break;
 
-  case 12:
+  case 12: /* line: '?' T_TIME  */
 #line 207 "ginsh_parser.ypp"
                                 {print_help("time");}
-#line 1509 "ginsh_parser.cpp"
+#line 1406 "ginsh_parser.cpp"
     break;
 
-  case 13:
+  case 13: /* line: '?' T_PRINT  */
 #line 208 "ginsh_parser.ypp"
                                 {print_help("print");}
-#line 1515 "ginsh_parser.cpp"
+#line 1412 "ginsh_parser.cpp"
     break;
 
-  case 14:
+  case 14: /* line: '?' T_IPRINT  */
 #line 209 "ginsh_parser.ypp"
                                 {print_help("iprint");}
-#line 1521 "ginsh_parser.cpp"
+#line 1418 "ginsh_parser.cpp"
     break;
 
-  case 15:
+  case 15: /* line: '?' T_PRINTLATEX  */
 #line 210 "ginsh_parser.ypp"
                                 {print_help("print_latex");}
-#line 1527 "ginsh_parser.cpp"
+#line 1424 "ginsh_parser.cpp"
     break;
 
-  case 16:
+  case 16: /* line: '?' T_PRINTCSRC  */
 #line 211 "ginsh_parser.ypp"
                                 {print_help("print_csrc");}
-#line 1533 "ginsh_parser.cpp"
+#line 1430 "ginsh_parser.cpp"
     break;
 
-  case 17:
+  case 17: /* line: '?' '?'  */
 #line 212 "ginsh_parser.ypp"
                                 {print_help_topics();}
-#line 1539 "ginsh_parser.cpp"
+#line 1436 "ginsh_parser.cpp"
     break;
 
-  case 18:
+  case 18: /* line: T_QUIT  */
 #line 213 "ginsh_parser.ypp"
                                 {YYACCEPT;}
-#line 1545 "ginsh_parser.cpp"
+#line 1442 "ginsh_parser.cpp"
     break;
 
-  case 19:
+  case 19: /* line: T_WARRANTY  */
 #line 214 "ginsh_parser.ypp"
                      {
 		cout << "This program is free software; you can redistribute it and/or modify it under\n";
@@ -1559,80 +1456,80 @@ yyreduce:
 		cout << "this program. If not, write to the Free Software Foundation,\n";
 		cout << "51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.\n";
 	}
-#line 1563 "ginsh_parser.cpp"
+#line 1460 "ginsh_parser.cpp"
     break;
 
-  case 20:
+  case 20: /* line: T_XYZZY  */
 #line 227 "ginsh_parser.ypp"
                                 {cout << "Nothing happens.\n";}
-#line 1569 "ginsh_parser.cpp"
+#line 1466 "ginsh_parser.cpp"
     break;
 
-  case 21:
+  case 21: /* line: T_INVENTORY  */
 #line 228 "ginsh_parser.ypp"
                                 {cout << "You're not carrying anything.\n";}
-#line 1575 "ginsh_parser.cpp"
+#line 1472 "ginsh_parser.cpp"
     break;
 
-  case 22:
+  case 22: /* line: T_LOOK  */
 #line 229 "ginsh_parser.ypp"
                                 {cout << "You're in a twisty little maze of passages, all alike.\n";}
-#line 1581 "ginsh_parser.cpp"
+#line 1478 "ginsh_parser.cpp"
     break;
 
-  case 23:
+  case 23: /* line: T_SCORE  */
 #line 230 "ginsh_parser.ypp"
                   {
 		cout << "If you were to quit now, you would score ";
 		cout << (syms.size() > 350 ? 350 : syms.size());
 		cout << " out of a possible 350.\n";
 	}
-#line 1591 "ginsh_parser.cpp"
+#line 1488 "ginsh_parser.cpp"
     break;
 
-  case 24:
+  case 24: /* line: T_REAL_SYMBOLS  */
 #line 235 "ginsh_parser.ypp"
                          { symboltype = domain::real; }
-#line 1597 "ginsh_parser.cpp"
+#line 1494 "ginsh_parser.cpp"
     break;
 
-  case 25:
+  case 25: /* line: T_COMPLEX_SYMBOLS  */
 #line 236 "ginsh_parser.ypp"
                             { symboltype = domain::complex; }
-#line 1603 "ginsh_parser.cpp"
+#line 1500 "ginsh_parser.cpp"
     break;
 
-  case 26:
+  case 26: /* $@1: %empty  */
 #line 237 "ginsh_parser.ypp"
                  { START_TIMER }
-#line 1609 "ginsh_parser.cpp"
+#line 1506 "ginsh_parser.cpp"
     break;
 
-  case 27:
+  case 27: /* line: T_TIME $@1 '(' exp ')'  */
 #line 237 "ginsh_parser.ypp"
                                              { STOP_TIMER PRINT_TIME_USED }
-#line 1615 "ginsh_parser.cpp"
+#line 1512 "ginsh_parser.cpp"
     break;
 
-  case 28:
+  case 28: /* line: error ';'  */
 #line 238 "ginsh_parser.ypp"
                                 {yyclearin; yyerrok;}
-#line 1621 "ginsh_parser.cpp"
+#line 1518 "ginsh_parser.cpp"
     break;
 
-  case 29:
+  case 29: /* line: error ':'  */
 #line 239 "ginsh_parser.ypp"
                                 {yyclearin; yyerrok;}
-#line 1627 "ginsh_parser.cpp"
+#line 1524 "ginsh_parser.cpp"
     break;
 
-  case 30:
+  case 30: /* exp: T_NUMBER  */
 #line 242 "ginsh_parser.ypp"
                                 {yyval = yyvsp[0];}
-#line 1633 "ginsh_parser.cpp"
+#line 1530 "ginsh_parser.cpp"
     break;
 
-  case 31:
+  case 31: /* exp: T_SYMBOL  */
 #line 243 "ginsh_parser.ypp"
                                 {
 		auto i = assigned_symbol_table.find(yyvsp[0]);
@@ -1641,46 +1538,46 @@ yyreduce:
 		else
 			yyval = i->second;
 	}
-#line 1645 "ginsh_parser.cpp"
+#line 1542 "ginsh_parser.cpp"
     break;
 
-  case 32:
+  case 32: /* exp: '\'' T_SYMBOL '\''  */
 #line 250 "ginsh_parser.ypp"
                                 {yyval = yyvsp[-1];}
-#line 1651 "ginsh_parser.cpp"
+#line 1548 "ginsh_parser.cpp"
     break;
 
-  case 33:
+  case 33: /* exp: T_LITERAL  */
 #line 251 "ginsh_parser.ypp"
                                 {yyval = yyvsp[0];}
-#line 1657 "ginsh_parser.cpp"
+#line 1554 "ginsh_parser.cpp"
     break;
 
-  case 34:
+  case 34: /* exp: T_DIGITS  */
 #line 252 "ginsh_parser.ypp"
                                 {yyval = yyvsp[0];}
-#line 1663 "ginsh_parser.cpp"
+#line 1560 "ginsh_parser.cpp"
     break;
 
-  case 35:
+  case 35: /* exp: T_QUOTE  */
 #line 253 "ginsh_parser.ypp"
                                 {yyval = exstack[0];}
-#line 1669 "ginsh_parser.cpp"
+#line 1566 "ginsh_parser.cpp"
     break;
 
-  case 36:
+  case 36: /* exp: T_QUOTE2  */
 #line 254 "ginsh_parser.ypp"
                                 {yyval = exstack[1];}
-#line 1675 "ginsh_parser.cpp"
+#line 1572 "ginsh_parser.cpp"
     break;
 
-  case 37:
+  case 37: /* exp: T_QUOTE3  */
 #line 255 "ginsh_parser.ypp"
                                 {yyval = exstack[2];}
-#line 1681 "ginsh_parser.cpp"
+#line 1578 "ginsh_parser.cpp"
     break;
 
-  case 38:
+  case 38: /* exp: T_SYMBOL '(' exprseq ')'  */
 #line 256 "ginsh_parser.ypp"
                                    {
 		auto i = find_function(yyvsp[-3], yyvsp[-1].nops());
@@ -1690,185 +1587,185 @@ yyreduce:
 			yyval = (i->second.p)(ex_to<exprseq>(yyvsp[-1]));
 		}
 	}
-#line 1694 "ginsh_parser.cpp"
+#line 1591 "ginsh_parser.cpp"
     break;
 
-  case 39:
+  case 39: /* exp: T_DIGITS '=' T_NUMBER  */
 #line 264 "ginsh_parser.ypp"
                                 {yyval = yyvsp[0]; Digits = ex_to<numeric>(yyvsp[0]).to_int();}
-#line 1700 "ginsh_parser.cpp"
+#line 1597 "ginsh_parser.cpp"
     break;
 
-  case 40:
+  case 40: /* exp: T_SYMBOL '=' exp  */
 #line 265 "ginsh_parser.ypp"
                                 {yyval = yyvsp[0]; assigned_symbol_table[yyvsp[-2]] = yyvsp[0]; }
-#line 1706 "ginsh_parser.cpp"
+#line 1603 "ginsh_parser.cpp"
     break;
 
-  case 41:
+  case 41: /* exp: exp T_EQUAL exp  */
 #line 266 "ginsh_parser.ypp"
                                 {yyval = yyvsp[-2] == yyvsp[0];}
-#line 1712 "ginsh_parser.cpp"
+#line 1609 "ginsh_parser.cpp"
     break;
 
-  case 42:
+  case 42: /* exp: exp T_NOTEQ exp  */
 #line 267 "ginsh_parser.ypp"
                                 {yyval = yyvsp[-2] != yyvsp[0];}
-#line 1718 "ginsh_parser.cpp"
+#line 1615 "ginsh_parser.cpp"
     break;
 
-  case 43:
+  case 43: /* exp: exp '<' exp  */
 #line 268 "ginsh_parser.ypp"
                                 {yyval = yyvsp[-2] < yyvsp[0];}
-#line 1724 "ginsh_parser.cpp"
+#line 1621 "ginsh_parser.cpp"
     break;
 
-  case 44:
+  case 44: /* exp: exp T_LESSEQ exp  */
 #line 269 "ginsh_parser.ypp"
                                 {yyval = yyvsp[-2] <= yyvsp[0];}
-#line 1730 "ginsh_parser.cpp"
+#line 1627 "ginsh_parser.cpp"
     break;
 
-  case 45:
+  case 45: /* exp: exp '>' exp  */
 #line 270 "ginsh_parser.ypp"
                                 {yyval = yyvsp[-2] > yyvsp[0];}
-#line 1736 "ginsh_parser.cpp"
+#line 1633 "ginsh_parser.cpp"
     break;
 
-  case 46:
+  case 46: /* exp: exp T_GREATEREQ exp  */
 #line 271 "ginsh_parser.ypp"
                                 {yyval = yyvsp[-2] >= yyvsp[0];}
-#line 1742 "ginsh_parser.cpp"
+#line 1639 "ginsh_parser.cpp"
     break;
 
-  case 47:
+  case 47: /* exp: exp '+' exp  */
 #line 272 "ginsh_parser.ypp"
                                 {yyval = yyvsp[-2] + yyvsp[0];}
-#line 1748 "ginsh_parser.cpp"
+#line 1645 "ginsh_parser.cpp"
     break;
 
-  case 48:
+  case 48: /* exp: exp '-' exp  */
 #line 273 "ginsh_parser.ypp"
                                 {yyval = yyvsp[-2] - yyvsp[0];}
-#line 1754 "ginsh_parser.cpp"
+#line 1651 "ginsh_parser.cpp"
     break;
 
-  case 49:
+  case 49: /* exp: exp '*' exp  */
 #line 274 "ginsh_parser.ypp"
                                 {yyval = yyvsp[-2] * yyvsp[0];}
-#line 1760 "ginsh_parser.cpp"
+#line 1657 "ginsh_parser.cpp"
     break;
 
-  case 50:
+  case 50: /* exp: exp '/' exp  */
 #line 275 "ginsh_parser.ypp"
                                 {yyval = yyvsp[-2] / yyvsp[0];}
-#line 1766 "ginsh_parser.cpp"
+#line 1663 "ginsh_parser.cpp"
     break;
 
-  case 51:
+  case 51: /* exp: '-' exp  */
 #line 276 "ginsh_parser.ypp"
                                 {yyval = -yyvsp[0];}
-#line 1772 "ginsh_parser.cpp"
+#line 1669 "ginsh_parser.cpp"
     break;
 
-  case 52:
+  case 52: /* exp: '+' exp  */
 #line 277 "ginsh_parser.ypp"
                                 {yyval = yyvsp[0];}
-#line 1778 "ginsh_parser.cpp"
+#line 1675 "ginsh_parser.cpp"
     break;
 
-  case 53:
+  case 53: /* exp: exp '^' exp  */
 #line 278 "ginsh_parser.ypp"
                                 {yyval = power(yyvsp[-2], yyvsp[0]);}
-#line 1784 "ginsh_parser.cpp"
+#line 1681 "ginsh_parser.cpp"
     break;
 
-  case 54:
+  case 54: /* exp: exp '!'  */
 #line 279 "ginsh_parser.ypp"
                                 {yyval = factorial(yyvsp[-1]);}
-#line 1790 "ginsh_parser.cpp"
+#line 1687 "ginsh_parser.cpp"
     break;
 
-  case 55:
+  case 55: /* exp: '(' exp ')'  */
 #line 280 "ginsh_parser.ypp"
                                 {yyval = yyvsp[-1];}
-#line 1796 "ginsh_parser.cpp"
+#line 1693 "ginsh_parser.cpp"
     break;
 
-  case 56:
+  case 56: /* exp: '{' list_or_empty '}'  */
 #line 281 "ginsh_parser.ypp"
                                 {yyval = yyvsp[-1];}
-#line 1802 "ginsh_parser.cpp"
+#line 1699 "ginsh_parser.cpp"
     break;
 
-  case 57:
+  case 57: /* exp: '[' matrix ']'  */
 #line 282 "ginsh_parser.ypp"
                                 {yyval = lst_to_matrix(ex_to<lst>(yyvsp[-1]));}
-#line 1808 "ginsh_parser.cpp"
+#line 1705 "ginsh_parser.cpp"
     break;
 
-  case 58:
+  case 58: /* exprseq: exp  */
 #line 285 "ginsh_parser.ypp"
                                 {yyval = exprseq{yyvsp[0]};}
-#line 1814 "ginsh_parser.cpp"
+#line 1711 "ginsh_parser.cpp"
     break;
 
-  case 59:
+  case 59: /* exprseq: exprseq ',' exp  */
 #line 286 "ginsh_parser.ypp"
                                 {exprseq es(ex_to<exprseq>(yyvsp[-2])); yyval = es.append(yyvsp[0]);}
-#line 1820 "ginsh_parser.cpp"
+#line 1717 "ginsh_parser.cpp"
     break;
 
-  case 60:
+  case 60: /* list_or_empty: %empty  */
 #line 289 "ginsh_parser.ypp"
                                 {yyval = *new lst;}
-#line 1826 "ginsh_parser.cpp"
+#line 1723 "ginsh_parser.cpp"
     break;
 
-  case 61:
+  case 61: /* list_or_empty: list  */
 #line 290 "ginsh_parser.ypp"
                                 {yyval = yyvsp[0];}
-#line 1832 "ginsh_parser.cpp"
+#line 1729 "ginsh_parser.cpp"
     break;
 
-  case 62:
+  case 62: /* list: exp  */
 #line 293 "ginsh_parser.ypp"
                                 {yyval = lst{yyvsp[0]};}
-#line 1838 "ginsh_parser.cpp"
+#line 1735 "ginsh_parser.cpp"
     break;
 
-  case 63:
+  case 63: /* list: list ',' exp  */
 #line 294 "ginsh_parser.ypp"
                                 {lst l(ex_to<lst>(yyvsp[-2])); yyval = l.append(yyvsp[0]);}
-#line 1844 "ginsh_parser.cpp"
+#line 1741 "ginsh_parser.cpp"
     break;
 
-  case 64:
+  case 64: /* matrix: '[' row ']'  */
 #line 297 "ginsh_parser.ypp"
                                 {yyval = lst{yyvsp[-1]};}
-#line 1850 "ginsh_parser.cpp"
+#line 1747 "ginsh_parser.cpp"
     break;
 
-  case 65:
+  case 65: /* matrix: matrix ',' '[' row ']'  */
 #line 298 "ginsh_parser.ypp"
                                  {lst l(ex_to<lst>(yyvsp[-4])); yyval = l.append(yyvsp[-1]);}
-#line 1856 "ginsh_parser.cpp"
+#line 1753 "ginsh_parser.cpp"
     break;
 
-  case 66:
+  case 66: /* row: exp  */
 #line 301 "ginsh_parser.ypp"
                                 {yyval = lst{yyvsp[0]};}
-#line 1862 "ginsh_parser.cpp"
+#line 1759 "ginsh_parser.cpp"
     break;
 
-  case 67:
+  case 67: /* row: row ',' exp  */
 #line 302 "ginsh_parser.ypp"
                                 {lst l(ex_to<lst>(yyvsp[-2])); yyval = l.append(yyvsp[0]);}
-#line 1868 "ginsh_parser.cpp"
+#line 1765 "ginsh_parser.cpp"
     break;
 
 
-#line 1872 "ginsh_parser.cpp"
+#line 1769 "ginsh_parser.cpp"
 
       default: break;
     }
@@ -2028,13 +1925,13 @@ yyabortlab:
 yyexhaustedlab:
   yyerror (YY_("memory exhausted"));
   yyresult = 2;
-  /* Fall through.  */
+  goto yyreturn;
 #endif
 
 
-/*-----------------------------------------------------.
-| yyreturn -- parsing is finished, return the result.  |
-`-----------------------------------------------------*/
+/*-------------------------------------------------------.
+| yyreturn -- parsing is finished, clean up and return.  |
+`-------------------------------------------------------*/
 yyreturn:
   if (yychar != YYEMPTY)
     {
